/**
 * @file
 * Attach the ResizeObserver to responsive images to load the best image style.
 */
(function (Drupal) {

  'use strict';

  Drupal.behaviors.easyResponsiveImages = {
    attach: function (context) {

      // Fetch all images containing a "data-srcset" attribute.
      const images = context.querySelectorAll('img[data-srcset]');

      // Return early if no images were found.
      if (!images || images.length === 0) {
        return;
      }

      // Find the best suitable image to display.
      const updateImage = function(image) {
        // Check if we need to apply a multiplier to the width check and load
        // bigger images.
        const multiplier = parseFloat(image.getAttribute('data-multiplier').replace('x',''));
        const multiplierMax = multiplier >= 1 ? multiplier : 1;
        const multiplierFactor = window.devicePixelRatio < multiplierMax ? window.devicePixelRatio : multiplierMax;

        // Check if we need to use the container height to select the best
        // image.
        const cover = image.hasAttribute('data-cover');

        // Check the available width for the image based on the multiplier.
        const imgWidth = Math.floor(image.clientWidth);
        const imgHeight = Math.floor(image.clientHeight);
        const parentWidth = Math.floor(image.parentNode.clientWidth);
        const parentHeight = Math.floor(image.parentNode.clientHeight);
        const availableWidth = parentWidth > imgWidth ? parentWidth : imgWidth;
        const availableHeight = parentHeight > imgHeight ? parentHeight : imgHeight;
        const targetWidth = multiplierFactor * availableWidth;
        const targetHeight = multiplierFactor * availableHeight;

        // Get the current width and height of the image.
        const attrWidth = image.getAttribute('width');
        const attrHeight = image.getAttribute('height');

        // Get all the source values and the current image source.
        const sources = image.getAttribute('data-srcset').split(',');
        const currentSrc = image.getAttribute('src');

        // Fetch the image ratio if it is available.
        let ratioWidth, ratioHeight;
        if (image.hasAttribute('data-ratio')) {
          [ratioWidth, ratioHeight] = image.getAttribute('data-ratio').split(':');
        }

        // If the selected image is already bigger than the available width,
        // we do not update the image.
        if (attrWidth && attrWidth > availableWidth && attrHeight && attrHeight > availableHeight) {
          return;
        }

        // Find the best matching source based on actual image space.
        let source, responsiveImgPath, responsiveImgWidth, responsiveImgHeight;
        for (source of sources) {
          let array = source.split(' ');
          responsiveImgPath = array[0];
          responsiveImgWidth = array[1].slice(0, -1);
          if (cover && parentWidth && parentHeight) {
            responsiveImgHeight = responsiveImgWidth * parentWidth / parentHeight;
          }
          else if (ratioWidth && ratioHeight) {
            responsiveImgHeight = responsiveImgWidth * ratioHeight / ratioWidth;
          }
          else if (attrWidth && attrHeight) {
            responsiveImgHeight = responsiveImgWidth * attrHeight / attrWidth;
          }
          // Stop looking if the image is bigger than the target width and the
          // image height is bigger than the target height.
          if (targetWidth < responsiveImgWidth && (!cover || !responsiveImgHeight || targetHeight < responsiveImgHeight)) {
            break;
          }
        }

        // If the image is already loaded, we do not update the image.
        if (responsiveImgPath === currentSrc) {
          return;
        }

        // Preload the new image and update the "src" when it is loaded. Also set
        // the "width" attribute to easily check if we need a new image after
        // resize.
        const img = new Image();
        img.src = responsiveImgPath;
        img.onload = function () {
          image.src = responsiveImgPath;
          // Temporarily calculate the height from the dimensions of the initial
          // width and height since this can also cause layout shifts.
          image.setAttribute('width', availableWidth);
          if (cover && parentWidth && parentHeight) {
            image.setAttribute('height', Math.round(availableWidth * parentHeight / parentWidth));
          }
          else if (ratioWidth && ratioHeight) {
            image.setAttribute('height', Math.round(availableWidth * ratioHeight / ratioWidth));
          }
          else if (attrWidth && attrHeight) {
            image.setAttribute('height', Math.round(availableWidth * attrHeight / attrWidth));
          }
        };

        // Update the image width/height onload to the actual image dimensions.
        if (cover || !ratioWidth || !ratioHeight) {
          image.onload = function () {
            this.setAttribute('width', Math.floor(this.width));
            this.setAttribute('height', Math.floor(this.height));
            this.setAttribute('data-loaded', true);
          };
        }
      };

      // Create a ResizeObserver to update the image "src" attribute when its
      // parent container resizes.
      const observer = new ResizeObserver(entries => {
        for (let entry of entries) {
          const images = entry.target.querySelectorAll('img[data-srcset]');
          images.forEach(image => {
            updateImage(image);
          });
        }
      });

      // Attach the ResizeObserver to the image containers.
      images.forEach(image => {
        // Make sure we only attach the observer once.
        if (image.hasAttribute('data-resizer')) {
          return;
        }

        // Mark the image that a resizer attached.
        image.setAttribute('data-resizer', '1');

        // Try to resize the image on load.
        updateImage(image);

        // Attach the observer to the image container.
        observer.observe(image.parentNode);
      });

    }
  };

})(Drupal);
